
/** @file vdif_chk is test reading VDIF header data 
 * @Version $Rev: 175 $
 * @Author M.Sekido
 * @Date  $Date: 2015-09-17 10:14:52 +0900 (木, 17  9月 2015) $
 * $Id: vdif_chk.cpp 175 2015-09-17 01:14:52Z vlbi $ 
 */

const char* version = "$Revision: 175 $ -Latest Update: $Date: 2015-09-17 10:14:52 +0900 (木, 17  9月 2015) $";

#include <iostream>
#include <string>
#include <vector>
using namespace std;

#include <cstdio>
#include <cstdlib>

#include "VirDumper.hpp"
#include "VDIF_Header.hpp"
#include "VDIF_Frame.hpp"
#include "DAS_info.hpp"
#include "VsiData.hpp"
#include "hist_g.hpp"
#include "conf_utl.hpp"

extern "C"{
#include <unistd.h>
}

/**! "main_cmn.hpp": Common Declraration of static parameters. 
 * Should be included in main file only. */
#include "main_cmn.hpp"

/** General Configulation data */
Config_set vdif_conf;

void usage(){
  cerr << "Usage: " << me << " -f <filename> [-d] [-n <int>] [-v <n>] [-g]\n";
  cerr << "        [-b <bit encode>] [-s <nframe>]\n";
  //  cerr << "-l : Display level 0-4\n";
  cerr << "-n : Number of frames to be displayed. (default:10), 0:all data.\n";
  cerr << "-d : Detailed output for every frame rather than every secnd.\n";
  cerr << "     (default: is print header by every second.) \n";
  cerr << "-v : Display info 0:simple(defalt),\n";
  cerr << "     1:EDVer RefEpoch,2:StnID,Thread No.,DataType,\n";
  cerr << "     3:same with 2 but one line, 4:Header with data.\n";
  cerr << "-s : Skip First <n> frames, in case of every frame(-d) option.\n";
  cerr << "-g : Histgram Display in case of every frame(-d).\n";
  //  cerr << "-e : Extended version Specification (NICT or none). Default is none\n";
  cerr << "-b  : Choice of bit encoding for histgram.\n";
  cerr << "      one of [ADS2000/ADS3000/SignMag/RevSign].(default=ADS2000)\n";
  cerr << "           ADS2000: 00,01,10,11 from bottom to top level.\n";
  cerr << "           ADS3000: 00,10,01,11 from bottom to top level.\n";
  cerr << "           SignMag: 01,00,10,11 from bottom to top level.\n";
  cerr << "           RevSgn : 10,00,01,11 from bottom to top level.\n";
  cerr << "  Ver.:" << version << endl;
  exit(1);
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */

  string fname;
  //  if(argc < 2) usage();
  //  int dlevel=1;
  //  VDIF_Ext_t ext_t = No_ext; // No VDIF-extended version
  //  size_t nsec = 0; // Endless display
  size_t ndisp = 10;
  bool every_frame = false;
  bool histgram_on = false;
  int nskip_frame(0);
  VsiData::BitEnc_t code_type = VsiData::ADS2000_2bit;
  int verbose_no=0;
  VDIF_Hdr::Prnt_t iedv(VDIF_Hdr::ONELINE);
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:b:dn:v:gs:")) { 
    case 'h':  usage(); break;
    case 'f':    fname = optarg;      break;
    case 's':    nskip_frame = strtol(optarg,0,10);      break;
    case 'g':    histgram_on = true;      break;
    case 'd':    every_frame = true;      break;
    case 'v':    verbose_no = strtol(optarg,0,10);      break;
    case 'b':
      if( strncmp(optarg, "ADS3000",7)==0) code_type = VsiData::ADS3000_2bit;
      if( strncmp(optarg, "SignMag",7)==0) code_type = VsiData::SignMag_2bit;
      if( strncmp(optarg, "RevSign",7)==0) code_type = VsiData::RevSgnMag_2bit;
      break;
    case 'n':    ndisp = atoi(optarg);      break;
      // case 'e':    if(strcmp(optarg,"NICT")==0) ext_t = NICT_ext; break;
      //    case 'l':    dlevel = atoi(optarg);      break;
      //    case 'd': vdif_hdr.Set_dbg(atoi(optarg)); break;
    case '?' : /* Error:  character not in list */ 
      fprintf(stderr, /* Yes */ 
	      "%s WARNING:  Unknown option on command line \n", me); 
      usage();
      break; 
    case ':' : /* Error:  missing parameter */ 
      fprintf(stderr, "%s WARNING:  Missing number on command line \n", me); 
      usage();
      break; 
    }
  }
  switch(verbose_no){
  case 1: iedv = VDIF_Hdr::ALL_ONE; break;
  case 2:iedv = VDIF_Hdr::ALL; break;
  case 3:iedv = VDIF_Hdr::CONST; break;
  case 4:iedv = VDIF_Hdr::ALL_ONE; break;
  case 0:
  default:
    iedv = VDIF_Hdr::ONELINE; break;
  }

  if(fname.empty()) usage();
  
  //  FILE *fp = fopen(fname.c_str(),"r");
  VDIF_Frame  vdif;
  if( ! vdif.ROpen(fname)){
    cerr << "Error in opening file: " << fname << endl;
    exit(1);
  }
  
  vector <VDIF_Hdr> hdrarr(10);
  //  uint32_t nframe = vdif.Read_every_sec_hdr(hdrarr, nsec);
  long nframe = vdif.Read_every_sec_hdr(hdrarr, 0);
  fprintf(stdout,"Total data length: %lu sec \n", hdrarr.size()); 

  if(nframe < 1){
    cerr << "E Failed to find VDIF Header.\n";
    exit(1);
  }
  
  long datsize = vdif.Framesize() - vdif.Headsize();
  int nch = hdrarr[0].Nch();
  int qbit = hdrarr[0].Qbit();
  int npt_frame = datsize * 8/qbit/nch; // #point per frame;
  const int Ncmp = 100000;
  const int Nframe_comp = npt_frame > Ncmp ? Ncmp : npt_frame;
  int lbound(0), ndiv(2), istep(0);
  switch(qbit){
  case 1: lbound = 0, istep=2; ndiv=2; break;
  case 2: lbound =-2; istep=2; ndiv=4;  break;
  case 4: lbound =-7, istep=1; ndiv=16; break;
  case 8: lbound =-127, istep=1; ndiv=256; break;
  }
  
  Hist_g <int8_t> hist( lbound, ndiv,istep);
  
  cout << "========= VDIF check ver.3.$Rev: 175 $ ================\n";
  double srate = (double)datsize * vdif.Nframe_sec() * 8/qbit/nch*1.e-6;
  fprintf(stdout," Reading Nsec =%ld sec, %lu frames, Rate(Estimate):%.3e Msps\n", 
	  hdrarr.size(), (unsigned long)nframe, srate); 
  //  string Info1 =  hdrarr[0].Str_hd( verbose_no );
  //  fprintf(stdout, "%s \n",Info1.c_str()); 
  if( every_frame){
    vdif.Rewind();
    vector <uint8_t> rbuf;
    DAS_Info das = vdif.DAS_Inf();// Sampling info
    // if( (vdif.Hdr().EDV() != EDV_NICT) &&
    // 	(vdif.Hdr().EDV() != EDV_NRAO)){
    //   uint32_t sr = srate*1000000;
    //   das.Set_Param(nch,qbit,sr);
    //   vdif.Set_DasInfo(das);
    // }

    VsiData vsidat(&das);
    vsidat.Make_Table(code_type); // Make Decode Table 
    ReturnFrameInfo retinfo;
    if(nskip_frame > 0) vdif.SeekFrames(nskip_frame);
    while(! vdif.EndofFile_Yes()){
      retinfo = vdif.Read_frame(hdrarr, rbuf, 1);
      if(retinfo.state == ReturnFrameInfo::NG ){
	fprintf(stdout, "E Failed to read frame\n"); break;
      }
      string Info1_dmy = hdrarr[0].Str_hd(iedv);
      //      if(Info1 != Info1_dmy) 
      fprintf(stdout, "%s",Info1_dmy.c_str()); 

      if(histgram_on){
	// Histgram Compute 
	//      vsidat.Interpret(rbuf,npt_frame);
	vsidat.Interpret(rbuf,Nframe_comp);
	if(nch > 7)fputc('\n',stdout);
	fprintf(stdout,"Hist[%%] ");
	for(int ich =0; ich<nch; ich++){
	  hist.Init();
	  hist.Accm(vsidat.ValArr(ich));
	  string str = hist.StrRatioArr(); 
	  fprintf(stdout,"(%2dch):%s ", ich, str.c_str());
	  if( (ich % 4 == 3) && ich != nch-1) fprintf(stdout,"\n\t");
	}
      }
      fprintf(stdout,"\n\t");
      //      fputc('\n', stdout);
      ndisp--;
      if(ndisp ==0) break;
    }
  }else{
    for(int i=0; i < (int)hdrarr.size(); i++){
	hdrarr[i].Print_hd(stdout, iedv);
	
	//fprintf(stdout," diff = %d\n", hdrarr[i].iFrame() - hdrarr[i-1].iFrame());
	//      if(ndisp !=0 && i > (int)ndisp) break;
    }
    
  }
  cout << "==========================================\n";

}


