/** @file  VLBI_Frame.hpp 
 *  A interface Class for VLBI Frame.
 *   This is succceded by K5/VSSP32, Mark5B, and VDIF frames
 * @Version $Rev: 198 $
 * @Author M.Sekido
 * @Date  $Date: 2016-08-10 10:59:09 +0900 (水, 10  8月 2016) $
 * $Id: VLBI_Frame.hpp 198 2016-08-10 01:59:09Z vlbi $ 
 */


#ifndef __VLBI_Frame_HPP__
#define __VLBI_Frame_HPP__

#include <cstring>
#include "DAS_info.hpp"
#include "Mk5B_Header.hpp"
#include "K5VSSP_Header.hpp"
#include "Trans_info.hpp"
#include "Logsystem.hpp"
//#include "k5vsi_mdl.hpp"
#include "crc.hpp"

extern "C"{
#include <time.h>
#include <errno.h>
}

#ifdef _USE_FOPEN64_
#define fopen(_Z,_X) fopen64(_Z,_X)
#endif
#ifdef _LARGEFILE_SOURCE                      // added on 2014-06-03
#define fseek(_Z,_X,_W) fseeko(_Z,_X,_W)      // added on 2014-06-03
#define ftell(_Z) ftello(_Z)                  // added on 2014-06-03
#endif                                        // added on 2014-06-03


/// Data Acquisition System Types
//enum DAS_t {K5VSSP, ADS2000, ADS1000, ADS3000, ADS3000plus, MK5B, Undef};

/** 
 * Frame size: 
 * VDIF Frame size must be multiple of 8bytes.
 * - IP header size = 24 bytes;
 * - UDP header size = 8 bytes
 * - SUDP  header size = 8byte
 * - VDIF header size = 32 bytes;
 * - Available space for application = 1500-32 = 1468
 * - Application Transport Protocol Header (ATPH) = 8 bytes
 * - Multiple of 8byte for VLBI_frame = 1456 at maximum.
 */

/// Function converting tm data to mjd
double tm2mjd(struct tm date);
/// Function for Computing MJD from year/moth/day
double cal2mjd(int day, int month, int year);

/** @class VLBI_Frame 
 * A Abstract interface class for general VLBI data-frames.
 *  VLBI_Frame as abstract interface class for general VLBI data-frames.
 */

class VLBI_Frame{  
public:
  /// VLBI Frame Types 
  enum VLBI_Frame_t {VDIF_frm, MK5B_frm, K5VSSP_frm, K5VSI_frm, Undef_frm};
protected:
    int32_t framesize;     ///< Frame (Including haeder) size (byte)
    //  int headsize;      ///< Header size (Bytes)
    DAS_Info das_inf;      ///< Sampling parameters  
    Time_dat data_epoch;   ///< Data Epoch 
    uint16_t  iframe;      ///< index of Frame
    uint32_t  nframe_sec;  /// add 2014/5/21 miya
public:
  /** Constructor */
  VLBI_Frame():framesize(0),iframe(0),nframe_sec(0) {}; /// add nframe_sec(10) 2014/5/21 miya
  /** Constructor */
  VLBI_Frame(const int fs):framesize(fs),iframe(0),nframe_sec(10){}; /// add nframe_sec(10) 2014/5/21 miya
  //  VLBI_Frame(const VLBI_Frame_t ft=VDIF_frm){Type_Set(ft);}
  /// Abstract function of Destructor 
  virtual ~VLBI_Frame(){ }; ///< Destructor 
  //  void Set_DAS_Info(DAS_Info& d){das_info = d;}
  //  int Type_Set(const VLBI_Frame_t ft); //< VLBI Frame type setting
  /** Set frame header and payload data (size byte) from data src */
  virtual int Make_frame( uint8_t* pkt, const uint8_t* src, const int size )=0;
  virtual const VLBI_Frame_t Type() const=0; ///< Return VLBI Frame type
  virtual string TypeName()=0; ///< Returns Frame type in string 
  /// Framesize (bytes) including header 
  int32_t Framesize()const{return framesize;}
  /** Set Sampling parameters, return data rate (Bytes/sec)  */
  virtual int32_t Set_Sparam(const int n, const int q, const uint32_t Hz);
  /// Abstract function to Return Headsize (byte) of the frame-type
  virtual int Headsize()const=0; // {return headsize;}
  //  void Set_Proto(VLBI_Proto* p){proto=p;}
  /// Set DAS information
  virtual const int Set_DasInfo(const DAS_Info& ds){das_inf = ds; return 0;}
  /// Returns reference of das info.
  const DAS_Info& DAS_Inf()const {return das_inf;}
  /// Abstract Function to set Epoch to the header 
  virtual void Set_Epoch(const Time_dat& t1){data_epoch=t1;}
  /// Abstract Function to set Epoch to the header 
  virtual void Set_Epoch(const time_t t1);
  /// Function to returns Epoch in Time_dat class.
  Time_dat Get_Epoch()const{return data_epoch;}
  virtual void Set_Epoch0(const time_t t1){;} ///< Do nothing except for VDIF
  /// Abstract Function to set Header and frame number to the packet 
  virtual int Set_header(const uint8_t* buf, const int i=-1){return 0;}
  /// Abstract Function to reset Frame counter number 
  virtual void Reset_header(){;}
  /// Abstract Function to set Framesize 
  virtual void Set_Framesize(const uint32_t sz)=0;
  /// Abstract Function to set Headr and DAS parameter to the packet.
  virtual void Set_Head_par(const time_t t1, DAS_Info* dasinf=0){
    Set_Epoch(t1); 
    if(dasinf != 0) das_inf = *dasinf; 
  }
  /// Abstract Function to set Station-name and Station-ID
  virtual void Set_StnameID(const char* name, const char* id=0)=0;
  /// Retrun Stname;
  virtual void Get_Stname(string & name)=0;
  /// Retrun Station ID;
  virtual void Get_StnID(string & id)=0;
  /// Retrun Number of Frames/sec; add 2014/5/20 miya
  const uint32_t Nframe_sec()const {return nframe_sec;}
  /// Set_Nframe_sec add 2014/5/22 miya
  void Set_Nframe_sec(const uint32_t ns) {nframe_sec=ns;};
};

inline  void VLBI_Frame::Set_Epoch(const time_t t1){
    struct tm tm1;
    time_t t2 = t1;
    gmtime_r(&t2, &tm1);
    int yy = tm1.tm_year + 1900;
    int mon = tm1.tm_mon + 1;
    int day = tm1.tm_mday;
    int h = tm1.tm_hour;
    int m = tm1.tm_min;
    int s = tm1.tm_sec;
    data_epoch.Set_time(yy,mon,day,h,m,s);
}


//========== K5VSSP ===from Here  ====================

class K5VSSP32_Frame: public VLBI_Frame{
    //  K5_VSI *k5vsi_dev;  //< K5_VSI Device  
    K5VSSP32_Hdr k5hdr;  ///<K5VSSP32 Hearder
    string stname;
    string stid;
public:
  /// Default constructor
  K5VSSP32_Frame():stname("Kashim34"),stid("kb"){Set_Nframe_sec(1);} ///Set_Nframe_sec add 2014/5/22 miya
  /// Returns VLBI Frame type by number
  const VLBI_Frame_t Type()const{return K5VSSP_frm;}
  /// Returns VLBI Frame type by string
  string TypeName(){return string("K5VSSP32");}
  /** Set up frame copying header and data
   * @param buf: buffer of the packet
   * @param src: data buffer
   * @param data_size: size of data in bytes
   */
  int Make_frame( uint8_t* buf, const uint8_t* src, const int data_size );
  /// Returns Headersize (bytes)
  int Headsize()const{return K5VSSP32_Hdr_size;}
  /// Set Sampling parameters
  int Set_Sparam(const int n, const int q, const uint32_t s);
  void Set_Epoch(const Time_dat& t1); ///< Set Epoch
  void Set_Epoch(const time_t t1); ///< Set Epoch
  /** Set Header info to the packet 
   * @param buf: buffer of the packet
   * @param ifrm: Frame number. If negative, just count up. 
   */
  int Set_header(const uint8_t* buf, const int ifrm=-1);
  /// Set Station name and Station ID
  void Set_StnameID(const char* name, const char* id=0){
	stname = name; k5hdr.Set_Stname(stname.c_str());
	if(id!=0){ stid = id;k5hdr.Set_StnID(id);}}
  /// Returns Pointer of Header data 
  const K5VSSP32_Hdr* pHeader()const{ return &k5hdr;}
  //  void Reset_header(){;} ///< Reset sequential number of header
  /// Set Framesize (bytes)
  void Set_Framesize(const uint32_t sz){;}
  /// Set Headr and DAS parameter to the packet.
  void Set_Head_par(const time_t t1, DAS_Info* dasinf=0);
  /// Set Das Info
  const int Set_DasInfo(const DAS_Info& ds);
  /// Retrun Stname;
  void Get_Stname(string & name){k5hdr.Get_Stname(name);}
  /// Retrun Station ID
  void Get_StnID(string & id) {k5hdr.Get_StnID(id);}
};

inline  void K5VSSP32_Frame::Set_Head_par(const time_t t1, DAS_Info* dasinf){
    VLBI_Frame::Set_Head_par(t1,dasinf);
    if(dasinf != 0){
	k5hdr.Set_Nch(dasinf->Nch());
	k5hdr.Set_Qbit(dasinf->Qbit());
	k5hdr.Set_Srate(dasinf->Srate()); // input in Hz
    }
}

inline int K5VSSP32_Frame::Make_frame( uint8_t* pkt,  const uint8_t* src,
				       const int  data_size){
    Set_header(pkt);
    pkt += K5VSSP32_Hdr_size;
    memcpy( pkt, src, data_size );
    return data_size + K5VSSP32_Hdr_size;
}

//========== K5VSSP ===to   Here  ====================

//========== K5VSI ===from Here  ====================

class K5VSI_Frame: public VLBI_Frame{
    //  K5_VSI *k5vsi_dev;  //< K5_VSI Device  
    //  K5VSSP32_Hdr k5hdr;  ///<K5VSSP32 Hearder
    string stname;
    //  string stid;
public:
    /// Default constructor
    K5VSI_Frame(){Set_Nframe_sec(0);} /// add :Set_Nframe_sec(0) 2014/5/22 miya
    /// Returns Frame type
    const VLBI_Frame_t Type()const{return K5VSI_frm;}
    string TypeName(){return string("K5VSI");}
    //  int Decompose_frame(const uint8_t* buf, uint32_t size ){return 0;}
    int Make_frame( uint8_t* buf, const uint8_t* src, const int size );
    /// Headersize (bytes)
    int Headsize()const{return 0;}
    //  int Set_Sparam(const int n, const int q, const uint32_t s);
    //  void Set_Epoch(const Time_dat& t1);
    //  void Set_Epoch(const time_t t1);
    //  int Set_header(const uint8_t* buf, const int i=-1){return 0;}
    //  void Set_StnameID(const char* name, const char* id=0){ stname = name; }
    //  void Reset_header(){;} ///< Reset sequential number of header
    //  const K5VSI32_Hdr* pHeader()const{ return &k5hdr;}
  void Set_Framesize(const uint32_t sz){;}
  /// Dummy Function to set Station-name and Station-ID
  void Set_StnameID(const char* name, const char* id=0){;}
  /// Dummy function of Retruning Stname;
  void Get_Stname(string & name){;}
  /// Dummy Function of Retruning Station ID
  void Get_StnID(string & id){;}
};


inline int K5VSI_Frame::Make_frame( uint8_t* pkt, const uint8_t* src,
				    const int data_size){
    //  Set_header(pkt);
    //  pkt += K5VSSP32_Hdr_size;
    memcpy( pkt, src, data_size );
    return data_size;
}

//========== K5VSI ===to   Here  ====================
#endif
